-- Raycast, 26/8/23

viewer.mode=STANDARD

function setup()
    assert(OrbitViewer, "Please include Cameras as a dependency")
    
    parameter.number("rayDia",0,10,5)
    parameter.color("rayColor",255,255,255)
    parameter.watch("myObj.entity.position")
    
    parameter.number("absdeltax",0,0.5,0.2, function () deltax=absdeltax end)
    
    white=color(255,255,255)
    
 --   fill(255)
    scene = craft.scene()
    scene.sun.rotation=quat.eulerAngles(20,45,-30)    
    v=scene.camera:add(OrbitViewer,vec3(0,0,0),100,0,1000)       
    
    skyMaterial=scene.sky.material
    skyMaterial.sky=color(79, 36, 244)
    skyMaterial.horizon=color(27, 24, 33)

    ambientIntensity=255   
    scene.ambientColor=color(ambientIntensity,ambientIntensity,ambientIntensity) 
    
    sunIntensity=1    
    light=scene.sun:get(craft.light)
    light.intensity=sunIntensity      
    
    n1=1  --index of vacuum
    n2=1.3 --index of sphere
    
    drawNormals=false
    
    makeCylinder()
    
--    function Object:init(shape, n, p, colour, opacity, size,rotation)
--    mySphere=Object("sphere",n2, vec3(-20,50,0), color(242, 51, 43), 0.3, 15)
--    mySphere=Object("cube",n2, vec3(-20,50,0), color(242, 51, 43), 0.3, vec3(15,10,20), vec3(1,1,0) )
    myObj=Object("model",n2, vec3(-20,50,0), color(242, 209, 43), 0.5, vec3(1,1,1)*15, vec3(1,1,0) )
    
    deltax=0.2
    
    --create rays with intial directions
    myray={}
    nray=5
    nSeg=5
    c=0
    sc=2
    for k=-nray,nray do 
    for j=-nray,nray  do
        c=c+1
        myray[c]={}
            
--            startPos=vec3(0,0,0)
--            endPos=startPos+vec3((-nray*0.5+j)/nray*2 ,1,0)*100
            
        startPos=vec3( k*sc, 0,  j*sc  )
        endPos=startPos+vec3(0,1,0)*100
            
        for i=1,nSeg do
            myray[c][i]=Ray(startPos, endPos,  rayColor)
        end        
    end
    end
    print("number of rays ",#myray)
end

function draw()
    update(DeltaTime)
    scene:draw()
end


function update(dt)

    if math.abs(myObj.entity.x)>30 then 
        deltax=-deltax
    end
    myObj.entity.x=myObj.entity.x+deltax

    for j=1,c do
        rayUpdate(j)
    end
    scene:update(dt)
end

function rayUpdate(j)
--    print("update ray",j)
    
    --set all segments inactive
    for i=1,nSeg do
        myray[j][i].entity.active=false
    end
    
    --raycast from origin
    ta=scene.physics:raycast(myray[j][1].startPos, myray[j][1].dir, 100)  
    if ta~=nil then
        
        ent=ta.entity
        entryPt=ta.point
        entryNor=ta.normal
        frac=ta.fraction
        index=ta.triangleIndex
        
        myray[j][1]:update(myray[j][1].startPos, entryPt, rayColor) 
        
        if drawNormals then 
        --draw entry normal
            norStart=entryPt-entryNor
            norEnd=entryPt+entryNor
            scene.debug:line(norStart,norEnd,white)  
        end
        
        --make internal ray
        i=(entryPt-myray[j][1].startPos):normalize()
        t=refractedRay(i,entryNor,n1,n2)
        if t==nil then 
            --print ("internally reflected") 
            return
        end
        
        --backwards raycast from outside       
        ta=scene.physics:raycast(entryPt+t*100, -t, 100)
        if ta~=nil then 
            exitPt=ta.point
            exitNor=ta.normal
        
            if drawNormals then 
                --draw exit normal
                norStart=exitPt-exitNor
                norEnd=exitPt+exitNor
                scene.debug:line(norStart,norEnd,white)  
            end
        
        --update internal ray
            myray[j][2]:update(entryPt,exitPt,rayColor)  
        --get exit ray
            i=(exitPt-entryPt):normalize()
            t=refractedRay(i,-exitNor,n2,n1)
            if t==nil then 
                --print ("invalid") 
                return 
            end
        --update exit ray
            --scene.debug:line(exitPt,exitPt+t*100, color(241, 44, 80))  
            myray[j][3]:update(exitPt,exitPt+t*50,rayColor)  
        end
    else
        --back to original ray
        myray[j][1]:update(myray[j][1].initialStartPos,myray[j][1].initialEndPos, rayColor) 
    end
        
end

